/*
 * Copyright 2016 Palantir Technologies, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
"use strict";
/*
 * This TS script reads the metadata from each TSLint built-in rule
 * and serializes it in a format appropriate for the docs website.
 *
 * This script expects there to be a tslint-gh-pages directory
 * parallel to the main tslint directory. The tslint-gh-pages should
 * have the gh-pages branch of the TSLint repo checked out.
 * One easy way to do this is with the following Git command:
 *
 * ```
 * git worktree add -b gh-pages ../tslint-gh-pages origin/gh-pages
 * ```
 *
 * See http://palantir.github.io/tslint/develop/docs/ for more info
 *
 */
var fs = require("fs");
var glob = require("glob");
var yaml = require("js-yaml");
var path = require("path");
var DOCS_DIR = "../docs";
process.chdir("./scripts");
/**
 * Documentation definition for rule modules.
 */
var ruleDocumentation = {
    dataFileName: "rules.json",
    exportName: "Rule",
    globPattern: "../lib/rules/*Rule.js",
    nameMetadataKey: "ruleName",
    pageGenerator: generateRuleFile,
    subDirectory: path.join(DOCS_DIR, "rules"),
};
/**
 * Documentation definition for formatter modules.
 */
var formatterDocumentation = {
    dataFileName: "formatters.json",
    exportName: "Formatter",
    globPattern: "../lib/formatters/*Formatter.js",
    nameMetadataKey: "formatterName",
    pageGenerator: generateFormatterFile,
    subDirectory: path.join(DOCS_DIR, "formatters"),
};
/**
 * Builds complete documentation.
 */
function buildDocumentation(documentation) {
    // Create each module's documentation file.
    var paths = glob.sync(documentation.globPattern);
    var metadataJson = paths.map(function (path) {
        return buildSingleModuleDocumentation(documentation, path);
    });
    // Create a data file with details of every module.
    buildDocumentationDataFile(documentation, metadataJson);
}
/**
 * Produces documentation for a single file/module.
 */
function buildSingleModuleDocumentation(documentation, modulePath) {
    // Load the module.
    // tslint:disable-next-line:no-var-requires
    var module = require(modulePath);
    var DocumentedItem = module[documentation.exportName];
    if (DocumentedItem != null && DocumentedItem.metadata != null) {
        // Build the module's page.
        var metadata = DocumentedItem.metadata;
        var fileData = documentation.pageGenerator(metadata);
        // Ensure a directory exists and write the module's file.
        var moduleName = metadata[documentation.nameMetadataKey];
        var fileDirectory = path.join(documentation.subDirectory, moduleName);
        if (!fs.existsSync(fileDirectory)) {
            fs.mkdirSync(fileDirectory);
        }
        fs.writeFileSync(path.join(fileDirectory, "index.html"), fileData);
        return metadata;
    }
}
function buildDocumentationDataFile(documentation, metadataJson) {
    var dataJson = JSON.stringify(metadataJson, undefined, 2);
    fs.writeFileSync(path.join(DOCS_DIR, "_data", documentation.dataFileName), dataJson);
}
/**
 * Generates Jekyll data from any item's metadata.
 */
function generateJekyllData(metadata, type, name) {
    return Object.assign({}, metadata, {
        layout: type.toLowerCase(),
        title: type + ": " + name,
    });
}
/**
 * Based off a rule's metadata, generates a Jekyll "HTML" file
 * that only consists of a YAML front matter block.
 */
function generateRuleFile(metadata) {
    var yamlData = generateJekyllData(metadata, "Rule", metadata.ruleName);
    yamlData.optionsJSON = JSON.stringify(metadata.options, undefined, 2);
    return "---\n" + yaml.safeDump(yamlData, { lineWidth: 140 }) + "---";
}
/**
 * Based off a formatter's metadata, generates a Jekyll "HTML" file
 * that only consists of a YAML front matter block.
 */
function generateFormatterFile(metadata) {
    var yamlData = generateJekyllData(metadata, "Formatter", metadata.formatterName);
    return "---\n" + yaml.safeDump(yamlData, { lineWidth: 140 }) + "---";
}
buildDocumentation(ruleDocumentation);
buildDocumentation(formatterDocumentation);
